# -*- coding: iso-8859-15 -*-
"""This module tries to convert the unicode accented chars into latex strings.
Will be probably rewritten to use pylatexenc.

This file is part of the physbiblio package.
"""
import re
import traceback
from pylatexenc.latexencode import utf8tolatex

try:
	from physbiblio.errors import pBLogger
except ImportError:
	print("Could not find physbiblio and its modules!")
	print(traceback.format_exc())
	raise

accents_changed = []


def parse_accents_str(string):
	"""Function that reads a string and translates
	all the known unicode characters into latex commands.

	Parameters:
		string: the string to be translated

	Output:
		the processed string
	"""
	if string is not None and string is not "":
		string = utf8tolatex(string, non_ascii_only=True)
	return string


def parse_accents_record(record):
	"""Function that reads the fields inside a bibtex dictionary
	and translates all the known unicode characters into latex commands.

	Parameters:
		record: the bibtex dictionary generated by bibtexparser

	Output:
		the dictionary after having processed all the fields
	"""
	for val in record:
		if val is not "ID" and len(record[val].strip()) > 0:
			tmp = utf8tolatex(record[val], non_ascii_only=True)
			if tmp != record[val]:
				pBLogger.info("    -> Converting bad characters in entry "
					+ "%s: "%record["ID"])
				pBLogger.info("         -- "+tmp.encode("utf-8"))
				accents_changed.append(record["ID"])
			record[val] = tmp
	return record

latex2Html_commands = [
	["textit","i"],
	["textbf","b"],
]
latex2Html_strings = [
	["\%","%"],
	["~"," "],
	["\ "," "],
]
latex_replace = [
	["text", "rm"],
]


def texToHtml(text):
	"""Function that converts some Latex commands into HTML commands.

	Parameters:
		text: the string to be processed

	Output:
		the processed string
	"""
	for tex, html in latex2Html_commands:
		match = re.compile('\\\\%s\{(.*| |\n)?\}'%tex, re.MULTILINE)
		for t in match.finditer(text):
			text = text.replace(
				t.group(),
				"<{html}>{cont}</{html}>".format(
					html=html, cont=t.group(1)))
	for tex, html in latex2Html_strings:
		text = text.replace(tex, html)
	for tex, new in latex_replace:
		match = re.compile('\\\\%s\{(.*| |\n)?\}'%tex, re.MULTILINE)
		for t in match.finditer(text):
			text = text.replace(t.group(), "\\%s{%s}"%(new, t.group(1)))
	return text



"""Module with the classes and functions that manage many generic
simple dialog windows of the PhysBiblio application.

This file is part of the physbiblio package.
"""
import sys
from PySide2.QtWidgets import \
	QDesktopWidget, QDialog, QFileDialog, QGridLayout, QInputDialog, \
	QMessageBox, QTextEdit, QPushButton

if sys.version_info[0] < 3:
	from StringIO import StringIO
else:
	from io import StringIO


def askYesNo(message, title="Question"):
	"""Uses `QMessageBox` to ask "Yes" or "No" for a given question.

	Parameters:
		message: the question to be displayed
		title: the window title

	Output:
		return True if the "Yes" button has been clicked,
			False otherwise
	"""
	mbox = QMessageBox(QMessageBox.Question, title, message)
	yesButton = mbox.addButton(QMessageBox.Yes)
	noButton = mbox.addButton(QMessageBox.No)
	mbox.setDefaultButton(noButton)
	mbox.exec_()
	if mbox.clickedButton() == yesButton:
		return True
	else:
		return False


def infoMessage(message, title="Information"):
	"""Uses `QMessageBox` to show a simple message.

	Parameters:
		message: the question to be displayed
		title: the window title
	"""
	mbox = QMessageBox(QMessageBox.Information, title, message)
	mbox.exec_()


class LongInfoMessage(QDialog):
	"""`infoMessage` version when a long text is expected"""

	def __init__(self, message, title="Information"):
		"""Class constructor.

		Parameters:
			message: the question to be displayed
			title: the window title
		"""
		QDialog.__init__(self)
		self.setWindowTitle(title)
		self.gridlayout = QGridLayout()
		self.textarea = QTextEdit(message)
		self.textarea.setReadOnly(True)
		self.gridlayout.addWidget(self.textarea, 0, 0, 1, 2)
		self.okbutton = QPushButton("OK", self)
		self.okbutton.clicked.connect(lambda: QDialog.close(self))
		self.gridlayout.addWidget(self.okbutton, 1, 1)
		self.setLayout(self.gridlayout)
		self.setGeometry(100, 100, 600, 400)
		qr = self.frameGeometry()
		cp = QDesktopWidget().availableGeometry().center()
		qr.moveCenter(cp)
		self.move(qr.topLeft())
		self.exec_()


def askGenericText(message, title, parent=None):
	"""Uses `QInputDialog` to ask a text answer for a given question.

	Parameters:
		message: the question to be displayed
		title: the window title
		parent (optional, default None): the parent of the window

	Output:
		return a tuple containing the text as first element
		and True/False
			(depending if the user selected "Ok" or "Cancel")
			as the second element
	"""
	dialog = QInputDialog(parent)
	dialog.setInputMode(QInputDialog.TextInput)
	dialog.setWindowTitle(title)
	dialog.setLabelText(message)
	out = dialog.exec_()
	return dialog.textValue(), out


def askFileName(parent=None,
		title="Filename to use:",
		filter="",
		dir=""):
	"""Uses `QFileDialog` to ask the name of a single, existing file

	Parameters (all optional):
		parent (default None): the parent of the window
		title: the window title
		filter: the filter to be used when displaying files
		dir: the initial directory

	Output:
		return the filename
		or an empty string if the user selected "Cancel"
	"""
	dialog = QFileDialog(parent, title, dir, filter)
	dialog.setFileMode(QFileDialog.ExistingFile)
	if dialog.exec_():
		fileNames = dialog.selectedFiles()
		try:
			return fileNames[0]
		except IndexError:
			return ""
	else:
		return ""


def askFileNames(parent=None,
		title="Filename to use:",
		filter="",
		dir=""):
	"""Uses `QFileDialog` to ask the names of a set of existing files

	Parameters (all optional):
		parent (default None): the parent of the window
		title: the window title
		filter: the filter to be used when displaying files
		dir: the initial directory

	Output:
		return the filenames list
		or an empty list depending if the user selected "Ok" or "Cancel"
	"""
	dialog = QFileDialog(parent, title, dir, filter)
	dialog.setFileMode(QFileDialog.ExistingFiles)
	dialog.setOption(QFileDialog.DontConfirmOverwrite, True)
	if dialog.exec_():
		fileNames = dialog.selectedFiles()
		return fileNames
	else:
		return []


def askSaveFileName(parent=None,
		title="Filename to use:",
		filter="",
		dir=""):
	"""Uses `QFileDialog` to ask the names of a single file
	where something will be saved (the file may not exist)

	Parameters (all optional):
		parent (default None): the parent of the window
		title: the window title
		filter: the filter to be used when displaying files
		dir: the initial directory

	Output:
		return the filename
		or an empty string depending if the user selected "Ok" or "Cancel"
	"""
	dialog = QFileDialog(parent, title, dir, filter)
	dialog.setFileMode(QFileDialog.AnyFile)
	dialog.setOption(QFileDialog.DontConfirmOverwrite, True)
	if dialog.exec_():
		fileNames = dialog.selectedFiles()
		try:
			return fileNames[0]
		except IndexError:
			return ""
	else:
		return ""


def askDirName(parent=None,
		title="Directory to use:",
		dir=""):
	"""Uses `QFileDialog` to ask the names of a single directory

	Parameters (all optional):
		parent (default None): the parent of the window
		title: the window title
		dir: the initial directory

	Output:
		return the directory name
		or an empty string depending if the user selected "Ok" or "Cancel"
	"""
	dialog = QFileDialog(parent, title, dir)
	dialog.setFileMode(QFileDialog.Directory)
	dialog.setOption(QFileDialog.ShowDirsOnly, True)
	if dialog.exec_():
		fileNames = dialog.selectedFiles()
		try:
			return fileNames[0]
		except IndexError:
			return ""
	else:
		return ""



"""Module that manages the command line calls of PhysBiblio.

This file is part of the physbiblio package.
"""
import sys
import datetime
import argparse

try:
	from physbiblio import __version__, __version_date__
	from physbiblio.config import pbConfig
except ImportError:
	print("Could not find physbiblio and its modules!")
	raise


def call_clean(args):
	"""Function used when the "clean" subcommand is called"""
	from physbiblio.database import pBDB
	pBDB.bibs.cleanBibtexs(startFrom=args.startFrom)
	pBDB.commit()


def call_cli(args):
	"""Function used when the "cli" subcommand is called"""
	from physbiblio.cli import cli as physBiblioCLI
	physBiblioCLI()


def call_export(args):
	"""Function used when the "export" subcommand is called"""
	from physbiblio.export import pBExport
	pBExport.exportAll(args.filename)


def call_tests(args):
	"""Function used when the "test" subcommand is called"""
	from physbiblio.setuptests import skipTestsSettings
	if args.database:
		skipTestsSettings.db = True
	if args.gui:
		skipTestsSettings.gui = True
	if args.long:
		skipTestsSettings.long = True
	if args.online_oai:
		skipTestsSettings.oai = True
	if args.online:
		skipTestsSettings.online = True
		skipTestsSettings.oai = True

	from physbiblio.testLoader import PBScanningLoader
	if sys.version_info[0] < 3:
		import unittest2 as unittest
	else:
		import unittest
	suite = PBScanningLoader().discover('physbiblio')
	testRunner = unittest.runner.TextTestRunner()
	testRunner.run(suite)


def call_tex(args):
	"""Function used when the "tex" subcommand is called"""
	from physbiblio.export import pBExport
	pBExport.exportForTexFile(
		args.texFiles,
		args.bibFile,
		overwrite=args.overwrite,
		autosave=args.save,
		updateExisting=args.updateExisting,
		removeUnused=args.removeUnused
		)


def call_update(args):
	"""Function used when the "update" subcommand is called"""
	from physbiblio.database import pBDB
	pBDB.bibs.searchOAIUpdates(startFrom=args.startFrom, force=args.force)
	pBDB.commit()


def call_oaiDates(date1, date2):
	"""Wrapper for getDailyInfoFromOAI + commit of the changes"""
	from physbiblio.database import pBDB
	pBDB.bibs.getDailyInfoFromOAI(date1, date2)
	pBDB.commit()


dateLast = datetime.date.today().strftime("%Y-%m-%d")


def call_dates(args):
	"""Use call_oaiDates to fetch updates between two dates
	specified in the command line
	"""
	call_oaiDates(args.start, args.end)


def call_daily(args):
	"""Use call_oaiDates to fetch daily updates"""
	call_oaiDates(
		(datetime.date.today() - datetime.timedelta(1)).strftime("%Y-%m-%d"),
		dateLast)


def call_weekly(args):
	"""Use call_oaiDates to fetch weekly updates"""
	call_oaiDates(
		(datetime.date.today() - datetime.timedelta(7)).strftime("%Y-%m-%d"),
		dateLast)


def call_gui(args=None):
	"""Function that runs the PhysBiblio GUI"""
	from PySide2.QtWidgets import QApplication
	try:
		from physbiblio.errors import pBLogger
		from physbiblio.database import pBDB
		from physbiblio.gui.mainWindow import MainWindow
		from physbiblio.gui.errorManager import pBGUIErrorManager
	except ImportError:
		print("Could not find physbiblio and its modules!")
		raise
	try:
		app = QApplication(sys.argv)
		mainWin = MainWindow()
		sys.excepthook = mainWin.errormessage.emit
		mainWin.show()
		mainWin.raise_()
		sys.exit(app.exec_())
	except NameError:
		pBLogger.critical("NameError:", exc_info=True)
	except SystemExit:
		pBDB.closeDB()
		pBLogger.info("Closing main window...")


class NewProfileAction(argparse.Action):
	"""Used to trigger a reload of the settings
	if a profile is specified as an argument
	"""

	def __init__(self, option_strings, dest, **kwargs):
		super(NewProfileAction, self).__init__(option_strings, dest, **kwargs)

	def __call__(self, parser, namespace, values, option_string=None):
		prof = values[0]
		if prof in pbConfig.profiles.keys():
			pbConfig.reloadProfiles(prof)
		setattr(namespace, self.dest, values)


def setParser():
	"""Contains the argparse definition of sub-commands and options"""
	parser = argparse.ArgumentParser(prog='PhysBiblio.exe')
	parser.add_argument(
		'-p',
		'--profile',
		action=NewProfileAction,
		nargs=1,
		choices=pbConfig.profiles.keys(),
		help='define the profile that must be used'
		)
	parser.add_argument(
		'-v',
		'--version',
		action='version',
		version='PhysBiblio %s (%s)'%(__version__, __version_date__)
		)
	subparsers = parser.add_subparsers(
		help='sub-command help',
		dest='cmd'
		)

	parser_clean = subparsers.add_parser(
		'clean',
		help='clean the entries in the database'
		)
	parser_clean.add_argument(
		'-s',
		'--startFrom',
		type=int,
		help='the index from which the cleaning should start',
		default=0
		)
	parser_clean.set_defaults(func=call_clean)

	parser_cli = subparsers.add_parser(
		'cli',
		help='open the internal command line interface'
		)
	parser_cli.set_defaults(func=call_cli)

	parser_daily = subparsers.add_parser(
		'daily',
		help='fetch the daily updates from INSPIRE-HEP OAI'
		)
	parser_daily.set_defaults(func=call_daily)

	parser_dates = subparsers.add_parser(
		'dates',
		help='fetch the updates from INSPIRE-HEP OAI between the given dates'
		)
	parser_dates.add_argument(
		'start',
		metavar='startdate',
		help='the starting date (format as yyyy-mm-dd)',
		default=(datetime.date.today() - datetime.timedelta(1))\
			.strftime("%Y-%m-%d")
		)
	parser_dates.add_argument(
		'end',
		metavar='enddate',
		help='the ending date (format as yyyy-mm-dd)',
		default=dateLast
		)
	parser_dates.set_defaults(func=call_dates)

	parser_export = subparsers.add_parser(
		'export',
		help='export all the entries in the database in a file'
		)
	parser_export.add_argument(
		'filename',
		help='the filename where to save the entries'
		)
	parser_export.set_defaults(func=call_export)

	parser_test = subparsers.add_parser(
		'test',
		help='run the test suite'
		)
	parser_test.add_argument(
		'-d',
		'--database',
		dest='database',
		action='store_true',
		help='do not perform database tests'
		)
	parser_test.add_argument(
		'-g',
		'--gui',
		dest='gui',
		action='store_true',
		help='do not perform gui tests'
		)
	parser_test.add_argument(
		'-l',
		'--long',
		dest='long',
		action='store_true',
		help='do not perform long tests'
		)
	parser_test.add_argument(
		'-o',
		'--online',
		dest='online',
		action='store_true',
		help='do not perform online tests'
		)
	parser_test.add_argument(
		'--online_oai',
		dest='online_oai',
		action='store_true',
		help='do not perform online tests with INSPIRE OAI'
		)
	parser_test.set_defaults(func=call_tests)

	parser_tex = subparsers.add_parser(
		'tex',
		help='read .tex file(s) and create a *.bib file '
			+ 'with the cited bibtexs')
	parser_tex.add_argument(
		'bibFile',
		metavar='bibfilename',
		help='the filename of the bib file where to write'
		)
	parser_tex.add_argument(
		'texFiles',
		metavar='texfilename',
		help='the filename of the tex file to read. '
			+ 'A folder or wildcards are admitted',
		nargs='+'
		)
	parser_tex.add_argument(
		'-o',
		'--overwrite',
		dest='overwrite',
		action='store_true',
		help='overwrite the bib file, if existing'
		)
	parser_tex.add_argument(
		'-r',
		'--removeUnused',
		dest='removeUnused',
		action='store_true',
		help='remove from the .bib file those elements '
			+ 'that are not more used in the tex'
		)
	parser_tex.add_argument(
		'-s',
		'--save',
		dest='save',
		action='store_false',
		help='save the changes in the database'
		)
	parser_tex.add_argument(
		'-u',
		'--updateExisting',
		dest='updateExisting',
		action='store_true',
		help='update the elements in the .bib file if they changed '
			+ 'in the database and remove duplicated entries'
		)
	parser_tex.set_defaults(func=call_tex)

	parser_update = subparsers.add_parser(
		'update',
		help='use INSPIRE to update the information in the database'
		)
	parser_update.add_argument(
		'-s',
		'--startFrom',
		type=int,
		help='the index from which the updating should start',
		default=0
		)
	parser_update.add_argument(
		'-f',
		'--force',
		action='store_true',
		help='force the update'
		)
	parser_update.set_defaults(func=call_update)

	parser_weekly = subparsers.add_parser(
		'weekly',
		help='fetch the weekly updates from INSPIRE-HEP OAI'
		)
	parser_weekly.set_defaults(func=call_weekly)

	parser_gui = subparsers.add_parser('gui', help='open the gui')
	parser_gui.set_defaults(func=call_gui)

	return parser



"""Module that opens the external links.

This file is part of the physbiblio package.
"""
import subprocess
import traceback
try:
	from physbiblio.errors import pBLogger
	from physbiblio.config import pbConfig
	from physbiblio.database import pBDB
	from physbiblio.pdf import pBPDF
except ImportError:
	print("Could not find physbiblio and its modules!")
	print(traceback.format_exc())
	raise


class ViewEntry():
	"""Contains methods to print or open a web link to the entry"""

	inspireRecord = pbConfig.inspireRecord
	inspireSearch = pbConfig.inspireSearchBase + "?p=find+"

	def __init__(self):
		"""Init the class, storing the name of
		the external web application
		and the base strings to build some links
		"""
		self.webApp = pbConfig.params["webApplication"]
		self.inspireRecord = pbConfig.inspireRecord
		self.inspireSearch = pbConfig.inspireSearchBase + "?p=find+"

	def getLink(self, key, arg="arxiv", fileArg=None):
		"""Uses database information to construct and
		print the web link, or the pdf module to open a pdf

		Parameters:
			key: the bibtex identifier as stored in the database.
				It may be a list (calls recursively itself)
			arg: the string defining the type of link/action
				("arxiv", "doi", "inspire", "file")
			fileArg: additional argument for the PDF module,
				used only if arg == "file"

		Output:
			link: the string of the web link
				if arg in ("arxiv", "doi", "inspire"),
				True if arg == "file",
				False if arg is invalid
		"""
		if isinstance(key, list):
			links = []
			for k in key:
				if arg is not "file":
					links.append(self.getLink(k, arg, fileArg))
				else:
					links.append(fileArg)
			return links
		arxiv = pBDB.bibs.getField(key, "arxiv")
		doi = pBDB.bibs.getField(key, "doi")
		inspire = pBDB.bibs.getField(key, "inspire")
		if arg is "arxiv" and arxiv:
			link = pBDB.bibs.getArxivUrl(key, "abs")
		elif arg is "doi" and doi:
			link = pBDB.bibs.getDoiUrl(key)
		elif arg is "inspire" and inspire:
			link = self.inspireRecord + inspire
		elif arg is "inspire" and arxiv:
			link = self.inspireSearch + arxiv
		elif arg is "inspire":
			link = self.inspireSearch + key
		elif arg is "file":
			pBPDF.openFile(key, fileArg)
			return fileArg
		else:
			pBLogger.warning("Invalid selection or missing information.\n"
				+ "Use one of (arxiv|doi|inspire|file) and "
				+ "check that all the information is available "
				+ "for entry '%s'."%key)
			return False

		return link

	def openLink(self, key, arg="arxiv", fileArg=None):
		"""Uses the getLink method to generate the web link
		and opens it in an external application

		Parameters:
			key, arg, fileArg as in the getLink method
		"""
		if isinstance(key, list):
			for k in key:
				self.openLink(k, arg, fileArg)
		else:
			if arg is "file":
				self.getLink(key, arg=arg, fileArg=fileArg)
				return
			elif arg is "link":
				link = key
			else:
				link = self.getLink(key, arg=arg, fileArg=fileArg)
			if link:
				if self.webApp != "":
					pBLogger.info("Opening '%s'..."%link)
					try:
						subprocess.Popen([self.webApp, link],
							stdout=subprocess.PIPE,
							stderr=subprocess.STDOUT)
					except OSError:
						pBLogger.warning("Opening link for '%s' failed!"%key)
			else:
				pBLogger.warning("Impossible to get the '%s'"%arg
					+ " link for entry '%s'"%key)


pBView = ViewEntry()



"""The package that contains all the modules used by the PhysBiblio,
including the `gui` subpackage.

This file is part of the physbiblio package.
"""
__author__ = 'Stefano Gariazzo'
__email__ = "stefano.gariazzo@gmail.com"

__version__ = "1.0.0"
__version_date__ = "18/02/2019"

__all__ = [
	"bibtexWriter",
	"cli",
	"config",
	"databaseCore",
	"database",
	"errors",
	"export",
	"inspireStats",
	"parseAccents",
	"pdf",
	"tablesDef",
	"view",
	"tests",
	"gui",
	"webimport",
	]



"""Module that uses INSPIRE-HEP to get author
(number of papers, h index, citations) and paper statistics (citations).

Uses matplotlib to do plots.

This file is part of the physbiblio package.
"""
import time
import os
import traceback
import os.path as osp
import json
import requests
import dateutil
import datetime
import matplotlib
matplotlib.use('Qt5Agg')
os.environ["QT_API"] = 'pyside2'
import matplotlib.pyplot as plt
import matplotlib.dates as mdates
from matplotlib.backends.backend_pdf import PdfPages

try:
	from physbiblio.errors import pBLogger
	from physbiblio.config import pbConfig
except ImportError:
	print("Could not find physbiblio and its modules!")
	print(traceback.format_exc())
	raise


class InspireStatsLoader():
	"""Class that contains the methods
	to collect information from INSPIRE-HEP
	"""

	urlBase = pbConfig.inspireSearchBase
	timeout = float(pbConfig.params["timeoutWebSearch"])
	authorStatsOpts = "&of=recjson&ot=recid,creation_date&so=a&rg=250"
	paperStatsOpts = "&of=recjson&ot=recid,creation_date&so=a&rg=250"
	skipPageOpt = "&jrec="
	maxPerPage = 250
	authorPlotInfo = {}
	paperPlotInfo = {}

	def __init__(self):
		"""The class constructor,
		defines some constants and search options
		"""
		self.urlBase = pbConfig.inspireSearchBase
		self.timeout = float(pbConfig.params["timeoutWebSearch"])
		self.authorPlotInfo = None
		self.paperPlotInfo = None
		self.allInfoA = {}
		self.authorPapersList = [[],[]]
		self.allCitations = []
		self.runningAuthorStats = True
		self.allInfoP = {}
		self.citingPapersList = [[],[]]

	def changeBackend(self, wantBackend):
		"""Changes the matplotlib backend currently in use.

		Parameters:
			wantBackend: a string that defines the wanted backend
		"""
		if wantBackend != matplotlib.get_backend():
			matplotlib.use(wantBackend, warn=False, force=True)
			from matplotlib import pyplot as plt
			pBLogger.info("Changed backend to %s"%matplotlib.get_backend())

	def JsonFromUrl(self, url):
		"""Function that downloads the url content
		and returns a Json object

		Parameters:
			url: string containing the url to be opened

		Output:
			the json object generated from the url content
		"""
		def getSeries(url):
			response = requests.get(url, timeout=self.timeout)
			try:
				return json.loads(response.content.decode("utf-8"))
			except ValueError:
				pBLogger.warning("Empty response!")
				return []
			except:
				pBLogger.exception("Cannot read the page content!")
				return []
		ser = 0
		complete = []
		while True:
			temp = getSeries(url + self.skipPageOpt + "%d"%(
				ser * self.maxPerPage + 1))
			if len(temp) < self.maxPerPage:
				return complete + temp
			else:
				complete += temp
				ser += 1

	def authorStats(self, authorName, plot=False, reset=True):
		"""Function that gets the data and
		constructs the statistics for a given author.

		Parameters:
			authorName: the author name as identified into INSPIRE-HEP,
				or a list of author names
				(it calls itself recursively for all the list elements)
			plot (boolean, default False): True to call self.plotStats
			reset (boolean, default False): True to delete
				all previous existing data
				(used as False when processing a list of authors)

		Output:
			a dictionary containing all the statistic information.
			For a single author, the structure is the following:
			{
				"name": the author name,
				"aI": The complete information,
					including the dictionaries with the single papers info
					(see self.paperStats), the citations
					and the corresponding dates,
				"paLi": a list of [id, date] of the papers
					associated with the author,
				"allLi": the complete list of [date, total citations]
					with all the citations to the papers,
				"meanLi": the complete list of
					[date, total citations/number of papers]
					computed at each point from "allLi" content,
				"h": the h-index,
				"figs" (only if `plot` is True): contains the figures.
					See self.plotStats
			}
		"""
		if reset:
			self.allInfoA = {}
			self.authorPapersList = [[],[]]
			self.allCitations = []
		if isinstance(authorName, list):
			for a in authorName:
				self.authorStats(a, reset=False)
			self.authorPlotInfo["name"] = authorName
			return self.authorPlotInfo
		pBLogger.info("Stats for author '%s'"%authorName)
		url = pbConfig.inspireSearchBase + "?p=author:" \
			+ authorName + self.authorStatsOpts
		data = self.JsonFromUrl(url)
		recid_authorPapers = [ "%d"%a["recid"] for a in data ]
		tot = len(recid_authorPapers)
		pBLogger.info("AuthorStats will process %d "%tot
			+ "total papers to retrieve citations")
		self.runningAuthorStats = True
		for i, p in enumerate(recid_authorPapers):
			if not self.runningAuthorStats:
				pBLogger.info("Received 'stop' signal. "
					+ "Interrupting download of information. "
					+ "Processing and exiting...")
				break
			if tot > 20:
				time.sleep(1)
			if p in self.allInfoA.keys(): continue
			self.allInfoA[p] = {}
			self.allInfoA[p]["date"] = dateutil.parser.parse(
				data[i]["creation_date"])
			self.authorPapersList[0].append(self.allInfoA[p]["date"])
			pBLogger.info("%5d / %d (%5.2f%%) - looking for paper: '%s'"%(
				i+1, tot, 100.*(i+1)/tot, p))
			paperInfo = self.paperStats(p,
				verbose=0, paperDate=self.allInfoA[p]["date"])
			self.allInfoA[p]["infoDict"] = paperInfo["aI"]
			self.allInfoA[p]["citingPapersList"] = paperInfo["citList"]
			for c,v in self.allInfoA[p]["infoDict"].items():
				self.allCitations.append(v["date"])
			pBLogger.info("")
		self.authorPapersList[1] = []
		for i,p in enumerate(sorted(self.authorPapersList[0])):
			self.authorPapersList[0][i] = p
			self.authorPapersList[1].append(i + 1)
		pBLogger.info("Saving citation counts...")
		allCitList = [[],[]]
		meanCitList = [[],[]]
		currPaper = 0
		for i,d in enumerate(sorted(self.allCitations)):
			if currPaper < len(self.authorPapersList[0])-1 \
					and d >= self.authorPapersList[0][currPaper+1]:
				currPaper += 1
			allCitList[0].append(d)
			allCitList[1].append(i+1)
			meanCitList[0].append(d)
			meanCitList[1].append((i+1.)/self.authorPapersList[1][currPaper])
		hind = 0
		citations = [len(self.allInfoA[k]["citingPapersList"][0]) - 2 \
			for k in self.allInfoA.keys()]
		for h in range(len(citations)):
			if len([a for a in citations if a >= h]) >= h:
				hind = h
		self.authorPlotInfo = {
			"name": authorName,
			"aI": self.allInfoA,
			"paLi": self.authorPapersList,
			"allLi": allCitList,
			"meanLi": meanCitList,
			"h": hind}
		if plot:
			self.authorPlotInfo["figs"] = self.plotStats(author=True)
		pBLogger.info("Stats for author '%s' completed!"%authorName)
		return self.authorPlotInfo

	def paperStats(self, paperID,
			plot=False, verbose=1, paperDate=None, reset=True):
		"""Function that gets the data and
		constructs the statistics for a given paper.

		Parameters:
			paperID (string): the INSPIRE-HEP id of the paper (a number)
			plot (boolean): whether or not the citations
				should be plotted (default False)
			verbose (int, default 1): increase the verbosity level
			paperDate (datetime, optional): the date of at which
				the paper was published
			reset (boolean, default False): True to delete
				all previous existing data
				(used as False when processing a list of IDs)

		Output:
			a dictionary containing all the desired information.
			The structure is the following:
			{
				"id": the paper ID,
				"aI": the list of creation date for all the papers,
					in INSPIRE-HEP order,
				"citList": the ordered list of citing papers,
				"fig" (only if `plot` is True): contains the figure.
					See self.plotStats
			}
		"""
		if reset:
			self.allInfoP = {}
			self.citingPapersList = [[],[]]
		if isinstance(paperID, list):
			for a in paperID:
				self.paperStats(a, reset=False)
			self.paperPlotInfo["id"] = paperID
			return self.paperPlotInfo
		if verbose > 0:
			pBLogger.info("Stats for paper '%s'"%paperID)
		url = pbConfig.inspireSearchBase + "?p=refersto:recid:" \
			+ paperID + self.paperStatsOpts
		data = self.JsonFromUrl(url)
		recid_citingPapers = [ a["recid"] for a in data ]
		if paperDate is not None:
			self.citingPapersList[0].append(paperDate)
		for i,p in enumerate(recid_citingPapers):
			self.allInfoP[p] = {}
			self.allInfoP[p]["date"] = dateutil.parser.parse(
				data[i]["creation_date"])
			self.citingPapersList[0].append(self.allInfoP[p]["date"])
		for i,p in enumerate(sorted(self.citingPapersList[0])):
			self.citingPapersList[0][i] = p
			self.citingPapersList[1].append(i+1)
		self.citingPapersList[0].append(
			datetime.datetime.fromordinal(datetime.date.today().toordinal()))
		try:
			self.citingPapersList[1].append(self.citingPapersList[1][-1])
		except IndexError:
			self.citingPapersList[1].append(0)
		self.paperPlotInfo = {
			"id": paperID,
			"aI": self.allInfoP,
			"citList": self.citingPapersList}
		if plot:
			self.paperPlotInfo["fig"] = self.plotStats(paper=True)
		if verbose > 0:
			pBLogger.info("Done!")
		return self.paperPlotInfo

	def plotStats(self,
			paper=False,
			author=False,
			show=False,
			save=False,
			path=".",
			markPapers=False,
			pickVal=6):
		"""Plot the collected information, using matplotlib.pyplot.

		Parameters:
			paper (boolean, default False): plot statistics
				for the last analyzed paper
			author (boolean, default False): plot statistics
				for the last analyzed author
			show (boolean, default False): True to show the plots
				in a separate window (with matplotlib.pyplot.show())
			save (boolean, default False): True to save the plots into files.
			path (string): where to save the plots
			markPapers (boolean, default False): True to draw
				a vertical lines at the dates
				corresponding to a paper appearing
			pickVal (float, default 6): the picker tolerance

		Output:
			False if paper==False and author==False,
			the matplotlib.pyplot figure containing
				the citation plot if paper==True,
			a list of matplotlib.pyplot figures containing
				the various plots if author==True
		"""
		if paper and self.paperPlotInfo is not None:
			if len(self.paperPlotInfo["citList"][0]) > 0:
				pBLogger.info(
					"Plotting for paper '%s'..."%self.paperPlotInfo["id"])
				fig, ax = plt.subplots()
				plt.plot(self.paperPlotInfo["citList"][0],
					self.paperPlotInfo["citList"][1], picker=pickVal)
				fig.autofmt_xdate()
				if save:
					pdf = PdfPages(
						osp.join(path, self.paperPlotInfo["id"]+'.pdf'))
					pdf.savefig()
					pdf.close()
				if show:
					plt.show()
				plt.close()
				return fig
		elif author and self.authorPlotInfo is not None:
			pBLogger.info(
				"Plotting for author '%s'..."%self.authorPlotInfo["name"])
			try:
				ymin = min(
					int(self.authorPlotInfo["allLi"][0][0].strftime("%Y"))-2,
					int(self.authorPlotInfo["paLi"][0][0].strftime("%Y"))-2)
				ymax = max(
					int(self.authorPlotInfo["allLi"][0][-1].strftime("%Y"))+2,
					int(self.authorPlotInfo["paLi"][0][-1].strftime("%Y"))+2)
			except:
				try:
					ymin = int(
						self.authorPlotInfo["paLi"][0][0].strftime("%Y"))-2
					ymax = int(
						self.authorPlotInfo["paLi"][0][-1].strftime("%Y"))+2
				except:
					pBLogger.warning("No publications for this author?")
					return False
			figs = []
			if len(self.authorPlotInfo["paLi"][0]) > 0:
				fig, ax = plt.subplots()
				plt.title("Paper number")
				plt.plot(self.authorPlotInfo["paLi"][0],
					self.authorPlotInfo["paLi"][1],
					picker=pickVal)
				fig.autofmt_xdate()
				if save:
					pdf = PdfPages(osp.join(
						path, self.authorPlotInfo["name"]+'_papers.pdf'))
					pdf.savefig()
					pdf.close()
				if show:
					plt.show()
				plt.close()
				figs.append(fig)

			if len(self.authorPlotInfo["paLi"][0]) > 0:
				fig, ax = plt.subplots()
				plt.title("Papers per year")
				ax.hist([int(q.strftime("%Y")) \
					for q in self.authorPlotInfo["paLi"][0]],
					bins=range(ymin, ymax), picker=True)
				ax.get_xaxis().get_major_formatter().set_useOffset(False)
				plt.xlim([ymin, ymax])
				if save:
					pdf = PdfPages(osp.join(path,
						self.authorPlotInfo["name"]+'_yearPapers.pdf'))
					pdf.savefig()
					pdf.close()
				if show:
					plt.show()
				plt.close()
				figs.append(fig)

			if len(self.authorPlotInfo["allLi"][0]) > 0:
				fig, ax = plt.subplots()
				plt.title("Total citations")
				plt.plot(self.authorPlotInfo["allLi"][0],
					self.authorPlotInfo["allLi"][1], picker=pickVal)
				fig.autofmt_xdate()
				if save:
					pdf = PdfPages(osp.join(path,
						self.authorPlotInfo["name"]+'_allCit.pdf'))
					pdf.savefig()
					pdf.close()
				if show:
					plt.show()
				plt.close()
				figs.append(fig)

			if len(self.authorPlotInfo["allLi"][0]) > 0:
				fig, ax = plt.subplots()
				plt.title("Citations per year")
				ax.hist([int(q.strftime("%Y")) \
					for q in self.authorPlotInfo["allLi"][0]],
					bins=range(ymin, ymax), picker=True)
				ax.get_xaxis().get_major_formatter().set_useOffset(False)
				plt.xlim([ymin, ymax])
				if save:
					pdf = PdfPages(osp.join(path,
						self.authorPlotInfo["name"]+'_yearCit.pdf'))
					pdf.savefig()
					pdf.close()
				if show:
					plt.show()
				plt.close()
				figs.append(fig)

			if len(self.authorPlotInfo["meanLi"][0]) > 0:
				fig, ax = plt.subplots()
				plt.title("Mean citations")
				plt.plot(self.authorPlotInfo["meanLi"][0],
					self.authorPlotInfo["meanLi"][1], picker=pickVal)
				fig.autofmt_xdate()
				if markPapers:
					for q in self.authorPlotInfo["paLi"][0]:
						plt.axvline(datetime.datetime(int(q.strftime("%Y")),
							int(q.strftime("%m")),
							int(q.strftime("%d"))),
							color='k',
							ls='--')
				if save:
					pdf = PdfPages(osp.join(path,
						self.authorPlotInfo["name"]+'_meanCit.pdf'))
					pdf.savefig()
					pdf.close()
				if show:
					plt.show()
				plt.close()
				figs.append(fig)

			if len(self.authorPlotInfo["aI"].keys()) > 0:
				fig, ax = plt.subplots()
				plt.title("Citations for each paper")
				for i,p in enumerate(self.authorPlotInfo["aI"].keys()):
					try:
						plt.plot(
							self.authorPlotInfo["aI"][p][
								"citingPapersList"][0],
							self.authorPlotInfo["aI"][p][
								"citingPapersList"][1])
					except:
						pBLogger.exception(
							"Something went wrong while plotting...")
				fig.autofmt_xdate()
				if save:
					pdf = PdfPages(osp.join(path,
						self.authorPlotInfo["name"]+'_paperCit.pdf'))
					pdf.savefig()
					pdf.close()
				if show:
					plt.show()
				plt.close()
				figs.append(fig)
			return figs
		else:
			pBLogger.info("Nothing to plot...")
			return False


pBStats = InspireStatsLoader()



"""Classes and functions that manage the export
of the database entries into .bib files.

This file is part of the physbiblio package.
"""
import os
import re
import shutil
import traceback
import codecs
import bibtexparser
from requests.structures import CaseInsensitiveDict

try:
	from physbiblio.errors import pBLogger
	from physbiblio.config import pbConfig
	from physbiblio.database import pBDB
	from physbiblio.bibtexWriter import pbWriter
except ImportError:
	print("Could not find physbiblio and its modules!")
	print(traceback.format_exc())
	raise


class PBExport():
	"""Class that contains the export functions and related."""

	exportForTexFlag = True
	backupExtension = ".bck"
	unwantedFields = ["owner", "timestamp", "__markedentry", "abstract"]

	def __init__(self):
		"""Initialize the class instance and set some default variables."""
		self.exportForTexFlag = True
		self.existingBibsList = None
		self.allCitations = set([])

	def backupCopy(self, fileName):
		"""Creates a backup copy of the given file.

		Parameters:
			fileName: the name of the file to be backed up
		"""
		if os.path.isfile(fileName):
			try:
				shutil.copy2(fileName, fileName + self.backupExtension)
			except IOError:
				pBLogger.exception(
					"Cannot write backup file.\nCheck the folder permissions.")
				return False
			else:
				return True
		return False

	def restoreBackupCopy(self, fileName):
		"""Restores the backup copy of the given file, if any.

		Parameters:
			fileName: the name of the file to be restore
		"""
		if os.path.isfile(fileName + self.backupExtension):
			try:
				shutil.copy2(fileName + self.backupExtension, fileName)
			except IOError:
				pBLogger.exception(
					"Cannot restore backup file.\nCheck the file permissions.")
				return False
			else:
				return True
		return False

	def rmBackupCopy(self, fileName):
		"""Deletes the backup copy of the given file, if any.

		Parameters:
			fileName: the name of the file of which
				the backup should be deleted
		"""
		if os.path.isfile(fileName + self.backupExtension):
			try:
				os.remove(fileName + self.backupExtension)
			except IOError:
				pBLogger.exception(
					"Cannot remove backup file.\nCheck the file permissions.")
				return False
			else:
				return True
		return True

	def exportLast(self, fileName):
		"""Export the last queried entries into a .bib file,
		if the list is not empty.

		Parameters:
			fileName: the name of the output bibtex file
		"""
		if pBDB.bibs.lastFetched:
			self.exportRows(fileName, pBDB.bibs.lastFetched)
		else:
			pBLogger.info("No last selection to export!")

	def exportAll(self, fileName):
		"""Export all the entries in the database into a .bib file.

		Parameters:
			fileName: the name of the output bibtex file
		"""
		pBDB.bibs.fetchAll(saveQuery=False, doFetch=False)
		self.exportRows(fileName, pBDB.bibs.fetchCursor())

	def exportSelected(self, fileName, rows):
		"""An alias for exportRows"""
		self.exportRows(fileName, rows)

	def exportRows(self, fileName, rows):
		"""Export the given entries into a .bib file.

		Parameters:
			fileName: the name of the output bibtex file
			rows: the list of entries to be exported
		"""
		self.backupCopy(fileName)
		if rows != []:
			try:
				with codecs.open(fileName, 'w', 'utf-8') as bibfile:
					for q in rows:
						bibfile.write(q["bibtex"] + "\n")
			except Exception:
				pBLogger.exception(
					"Problems in exporting .bib file!", traceback)
				self.restoreBackupCopy(fileName)
		else:
			pBLogger.info("No elements to export!")
		self.rmBackupCopy(fileName)

	def exportForTexFile(self,
			texFileName,
			outFileName,
			overwrite=False,
			autosave=True,
			updateExisting=False,
			removeUnused=False,
			newOperation=True):
		"""Reads a .tex file looking for the \cite{} commands,
		collects the bibtex entries cited in the text and
		stores them in a bibtex file.
		The entries are taken from the database first,
		or from INSPIRE-HEP if possible.
		The downloaded entries are saved in the database.

		Parameters:
			texFileName: the name (or a list of names)
				of the considered .tex file(s)
			outFileName: the name of the output file,
				where the required entries will be added
			overwrite (boolean, default False):
				if True, the previous version of the file is replaced
				and no backup copy is created
			autosave (boolean, default True):
				if True, the changes to the database are automatically saved.
			updateExisting (boolean, default False):
				if True, remove duplicates and update entries
				that have been chenged in the DB
			removeUnused (boolean, default False):
				if True, remove bibtex entries that are no more cited
				in the tex files
			newOperation (boolean, default True):
				reset the self.existingBibsList and read file .bib content.
				Time consuming! better to just keep it updated
				when using multiple texs...

		Output:
			True if successful, False if errors occurred
		"""
		db = bibtexparser.bibdatabase.BibDatabase()

		def printOutput(reqBibkeys,
				miss,
				retr,
				nFound,
				unexp,
				nKeys,
				warn,
				totalCites,
				full=False):
			"""Print information on the process"""
			pBLogger.info("\n\nRESUME")
			if totalCites is not None:
				pBLogger.info("%d keys found in .tex file"%totalCites)
			pBLogger.info("%d new keys found in .tex file"%len(reqBibkeys))
			j = ', '
			if full:
				pBLogger.info(j.join(reqBibkeys))
			if len(miss) > 0:
				pBLogger.info(
					"%d required entries were missing in database"%len(miss))
				if full:
					pBLogger.info(j.join(miss))
			if len(retr) > 0:
				pBLogger.info("%d new entries retrieved:"%len(retr))
				pBLogger.info(j.join(retr))
			if len(nFound) > 0:
				pBLogger.info("Impossible to find %d entries:"%len(nFound))
				pBLogger.info(j.join(nFound))
			if len(unexp) > 0:
				pBLogger.info("Unexpected errors for %d entries:"%len(unexp))
				pBLogger.info(j.join(unexp))
			if len(nKeys.keys()) > 0:
				pBLogger.info(
					"Possible non-matching keys in %d entries:\n"%len(
						nKeys.keys()) \
					+ "\n".join(["'%s' => '%s'"%(k, n) \
						for k, n in nKeys.items() ] ) )
			pBLogger.info("     %s warning(s) occurred!"%warn)

		def saveEntryOutBib(a, m=None):
			"""Remove unwanted fields and add the bibtex entry
			to the output file

			Parameters:
				a: the bibtex entry
				m: the ID (bibtex key) of the entry,
					if it is not the default one
			"""
			entry = bibtexparser.bparser.BibTexParser(common_strings=True
				).parse(a).entries[0]
			for u in self.unwantedFields:
				try:
					del entry[u]
				except KeyError:
					pass
			if m is not None:
				m = m.strip()
				if m != entry["ID"].strip():
					entry["ID"] = m
			db.entries = [entry]
			bibf = pbWriter.write(db)
			try:
				with open(outFileName, "a") as o:
					o.write(bibf)
					pBLogger.info("%s inserted in output file"%m)
			except IOError:
				pBLogger.exception("Impossible to write file '%s'"%outFileName)
				return False

		def removeUnusedBibtexs(existingBibsDict):
			"""Functions that reads the list of bibtex entries
			in the existing .bib file and removes
			the ones that are not inside \cite commands
			"""
			newDict = {}
			notFound = []
			for k, v in existingBibsDict.items():
				if k in self.allCitations:
					newDict[k] = existingBibsDict[k]
				else:
					notFound.append(k)
			db.entries = [newDict[k] for k in sorted(
				[e["ID"] for e in newDict.values()],
				key = lambda s: s.lower())]
			bibf = pbWriter.write(db)
			try:
				with open(outFileName, "w") as o:
					o.write("%file written by PhysBiblio\n" + bibf)
					pBLogger.info(
						"Output file updated. Removed entries:\n%s"%notFound)
			except IOError:
				pBLogger.exception("Impossible to write file '%s'"%outFileName)

		self.exportForTexFlag = True
		pBLogger.info("Starting exportForTexFile...\n\n")
		pBLogger.info("Reading keys from '%s'"%texFileName)
		pBLogger.info("Saving in '%s'"%outFileName)
		if autosave:
			pBLogger.info("Changes will be automatically saved at the end!")

		missing = []
		newKeys = {}
		notFound = []
		requiredBibkeys = []
		retrieved = []
		unexpected = []
		warnings = 0
		totalCites = 0

		#if overwrite, reset the output file
		if overwrite:
			updateExisting = False
			removeUnused = False
			try:
				with open(outFileName, "w") as o:
					o.write("%file written by PhysBiblio\n")
			except IOError:
				pBLogger.exception(
					"Cannot write on file.\nCheck the file permissions.")
				return False

		#read previous content of output file, if any
		try:
			with open(outFileName, "r") as f:
				existingBibText = f.read()
		except IOError:
			pBLogger.error(
				"Cannot read file %s.\nCreating one."%outFileName)
			try:
				open(outFileName, "w").close()
			except IOError:
				pBLogger.exception(
					"Cannot create file %s!"%outFileName)
				return False
			existingBibText = ""

		# this is time consuming if there are many entries.
		# Do not load it every time for multiple texs!
		if newOperation:
			self.allCitations = set([])
			if existingBibText != "":
				self.existingBibsList = pBDB.bibs.parseAllBibtexs(
					existingBibText, verbose=False)
			else:
				self.existingBibsList = []
		# work with dictionary, so that if there are repeated entries
		# (entries with same ID) they are automatically discarded
		existingBibsDict = CaseInsensitiveDict()
		for e in self.existingBibsList:
			existingBibsDict[e["ID"]] = e

		#if requested, do some cleaning
		if updateExisting:
			#update entry from DB if existing
			for k, v in existingBibsDict.items():
				e = pBDB.bibs.getByBibtex(k, saveQuery=False)
				if len(e)>0 and e[0]["bibtexDict"] != v:
					existingBibsDict[k] = e[0]["bibtexDict"]
					if existingBibsDict[k]["ID"].lower() != k.lower():
						existingBibsDict[k]["ID"] = k

			# write new (updated) bib content
			# (so also repeated entries are removed)
			db.entries = [existingBibsDict[k] for k in sorted(
				[e["ID"] for e in existingBibsDict.values()],
				key = lambda s: s.lower())]
			bibf = pbWriter.write(db)
			try:
				with open(outFileName, "w") as o:
					o.write("%file written by PhysBiblio\n" + bibf)
					pBLogger.info("Output file updated")
			except IOError:
				pBLogger.exception("Impossible to write file '%s'"%outFileName)

		# if there is a list of tex files, run this function
		# for each of them...no updateExisting and removeUnused!
		if isinstance(texFileName, list):
			if len(texFileName) == 0:
				return False
			elif len(texFileName) == 1:
				texFileName = texFileName[0]
			else:
				for t in texFileName:
					req, m, ret, nF, un, nK, w, cits = self.exportForTexFile(
						t,
						outFileName,
						overwrite=False,
						autosave=autosave,
						updateExisting=False,
						removeUnused=False,
						newOperation=False)
					requiredBibkeys += req
					missing += m
					retrieved += ret
					notFound += nF
					unexpected += un
					for k, v in nK.items():
						newKeys[k] = v
					warnings += w
				pBLogger.info("Done for all the texFiles.\n\n")
				if removeUnused:
					removeUnusedBibtexs(existingBibsDict)
				printOutput(requiredBibkeys,
					missing,
					retrieved,
					notFound,
					unexpected,
					newKeys,
					warnings,
					len(self.allCitations),
					full=True)
				return requiredBibkeys, missing, retrieved, notFound, \
					unexpected, newKeys, warnings, len(self.allCitations)

		#read the texFile
		keyscont=""
		try:
			with open(texFileName) as r:
				keyscont += r.read()
		except IOError:
			pBLogger.exception("The file %s does not exist."%texFileName)
			return False

		#extract \cite* commands
		matchKeys = '([0-9A-Za-z_\-\':]+)'
		cite = re.compile('\\\\(cite|citep|citet)\{([\n ]*' \
			+ matchKeys + '[,]?[\n ]*)*\}', re.MULTILINE)	#find \cite{...}
		citeKeys = re.compile(matchKeys,
			re.MULTILINE)	#find the keys inside \cite{...}
		citaz = [ m for m in cite.finditer(keyscont) if m != "" ]
		pBLogger.info(r"%d \cite commands found in .tex file"%len(citaz))

		#extract required keys from \cite* commands
		for c in citaz:
			try:
				for e in [l.group(1) for l in citeKeys.finditer(c.group())]:
					e = e.strip()
					if e == "" or e in ["cite", "citep", "citet"]:
						continue
					self.allCitations.add(e)
					if e not in requiredBibkeys:
						try:
							#this it's just to check if already present
							tmp = existingBibsDict[e]
						except KeyError:
							requiredBibkeys.append(e)
			except (IndexError, AttributeError, TypeError):
				pBLogger.warning(
					"Cannot recognize citation list in '%s'!"%c.group())
				a = []
		pBLogger.info("%d new keys found, %d in total."%(
			len(requiredBibkeys), len(self.allCitations)))

		#if True, remove unused bibtex entries
		if removeUnused:
			removeUnusedBibtexs(existingBibsDict)

		# check what is missing in the database and insert/import
		# what is needed:
		for m in requiredBibkeys:
			if m.strip() == "":
				continue
			entry = pBDB.bibs.getByBibtex(m)
			entryMissing = len(entry) == 0
			if not self.exportForTexFlag:
				#if flag set, stop execution and
				#go to the end skipping everything
				continue
			elif not entryMissing:
				#if already in the database, just insert it as it is
				bibtex = entry[0]["bibtex"]
				bibtexDict = entry[0]["bibtexDict"]
			else:
				#if no entry is found, mark it as missing
				missing.append(m)
				#if not present, try INSPIRE import
				pBLogger.info(
					"Key '%s' missing, trying to import it from Web"%m)
				newWeb = pBDB.bibs.loadAndInsert(m, returnBibtex=True)
				newCheck = pBDB.bibs.getByBibtex(m, saveQuery=False)

				#if the import worked, insert the entry
				if len(newCheck) > 0:
					#if key is not matching,
					#just replace it in the exported bib and print a message
					if m.strip().lower() != newCheck[0]["bibkey"].lower():
						warnings += 1
						newKeys[m] = newCheck[0]["bibkey"]
					if newCheck[0]["bibkey"] not in retrieved:
						retrieved.append(newCheck[0]["bibkey"])
					pBDB.catBib.insert(
						pbConfig.params["defaultCategories"],
						newCheck[0]["bibkey"])
					bibtex = newCheck[0]["bibtex"]
					bibtexDict = newCheck[0]["bibtexDict"]
				else:
					#if nothing found, add a warning for the end
					warnings += 1
					notFound.append(m)
					continue
				pBLogger.info("\n")
			#save in output file
			try:
				bibtexDict["ID"] = m
				self.existingBibsList.append(bibtexDict)
				saveEntryOutBib(bibtex, m)
			except:
				unexpected.append(m)
				pBLogger.exception("Unexpected error in extracting entry "
					+ "'%s' to the output file"%m)

		if autosave:
			pBDB.commit()
		printOutput(requiredBibkeys, missing, retrieved, notFound,
			unexpected, newKeys, warnings, len(self.allCitations))
		return requiredBibkeys, missing, retrieved, notFound, \
			unexpected, newKeys, warnings, len(self.allCitations)

	def updateExportedBib(self, fileName, overwrite=False):
		"""Reads a bibtex file and updates the entries that it contains,
		for example if the entry has been published.

		Parameters:
			fileName: the name of the considered bibtex file
			overwrite (boolean, default False): if True,
				the previous version of the file is replaced
				and no backup copy is created

		Output:
			True if successful, False if errors occurred
		"""
		self.backupCopy(fileName)
		bibfile=""
		try:
			with open(fileName) as r:
				bibfile += r.read()
		except IOError:
			pBLogger.exception(
				"Cannot write on file.\nCheck the file permissions.")
			return False
		try:
			biblist = bibtexparser.bparser.BibTexParser(common_strings=True
				).parse(bibfile)
		except IndexError:
			pBLogger.exception("Problems in loading the .bib file!")
			return False
		db = bibtexparser.bibdatabase.BibDatabase()
		db.entries = []
		for b in biblist.entries:
			key = b["ID"]
			element = pBDB.bibs.getByBibkey(key, saveQuery=False)
			if len(element)>0:
				db.entries.append(element[0]["bibtexDict"])
			else:
				db.entries.append(b)
		txt = pbWriter.write(db).strip()
		try:
			with codecs.open(fileName, 'w', 'utf-8') as outfile:
				outfile.write(txt)
		except Exception:
			pBLogger.exception("Problems in exporting .bib file!")
			self.restoreBackupCopy(fileName)
			return False
		if overwrite:
			self.rmBackupCopy(fileName)
		return True


pBExport = PBExport()
